@@ -58,7 +58,7 @@ static void terminate(void)
 	DEBUG(0,("Got SIGTERM: going down...\n"));
   
 	/* Write out wins.dat file if samba is a WINS server */
-	wins_write_database(False);
+	wins_write_database(0,False);
   
 	/* Remove all SELF registered names from WINS */
 	release_wins_names();
@@ -76,7 +76,8 @@ static void terminate(void)
  Handle a SHUTDOWN message from smbcontrol.
  **************************************************************************** */
 
-static void nmbd_terminate(int msg_type, pid_t src, void *buf, size_t len)
+static void nmbd_terminate(int msg_type, struct process_id src,
+			   void *buf, size_t len, void *private_data)
 {
 	terminate();
 }
@@ -105,55 +106,13 @@ static void sig_hup(int sig)
 	sys_select_signal(SIGHUP);
 }
 
-#if DUMP_CORE
-/**************************************************************************** **
- Prepare to dump a core file - carefully!
- **************************************************************************** */
-
-static BOOL dump_core(void)
-{
-	char *p;
-	pstring dname;
-	pstrcpy( dname, lp_logfile() );
-	if ((p=strrchr_m(dname,'/')))
-		*p=0;
-	pstrcat( dname, "/corefiles" );
-	mkdir( dname, 0700 );
-	sys_chown( dname, getuid(), getgid() );
-	chmod( dname, 0700 );
-	if ( chdir(dname) )
-		return( False );
-	umask( ~(0700) );
-
-#ifdef HAVE_GETRLIMIT
-#ifdef RLIMIT_CORE
-	{
-		struct rlimit rlp;
-		getrlimit( RLIMIT_CORE, &rlp );
-		rlp.rlim_cur = MAX( 4*1024*1024, rlp.rlim_cur );
-		setrlimit( RLIMIT_CORE, &rlp );
-		getrlimit( RLIMIT_CORE, &rlp );
-		DEBUG( 3, ( "Core limits now %d %d\n", (int)rlp.rlim_cur, (int)rlp.rlim_max ) );
-	}
-#endif
-#endif
-
-
-	DEBUG(0,("Dumping core in %s\n",dname));
-	abort();
-	return( True );
-}
-#endif
-
 /**************************************************************************** **
  Possibly continue after a fault.
  **************************************************************************** */
 
 static void fault_continue(void)
 {
-#if DUMP_CORE
 	dump_core();
-#endif
 }
 
 /**************************************************************************** **
@@ -191,27 +150,39 @@ static void expire_names_and_servers(time_t t)
 
 /************************************************************************** **
  Reload the list of network interfaces.
+ Doesn't return until a network interface is up.
  ************************************************************************** */
 
-static BOOL reload_interfaces(time_t t)
+static void reload_interfaces(time_t t)
 {
 	static time_t lastt;
 	int n;
 	struct subnet_record *subrec;
 
-	if (t && ((t - lastt) < NMBD_INTERFACES_RELOAD)) return False;
+	if (t && ((t - lastt) < NMBD_INTERFACES_RELOAD)) {
+		return;
+	}
 	lastt = t;
 
-	if (!interfaces_changed()) return False;
+	if (!interfaces_changed()) {
+		return;
+	}
 
 	/* the list of probed interfaces has changed, we may need to add/remove
 	   some subnets */
 	load_interfaces();
 
+  try_again:
+
 	/* find any interfaces that need adding */
 	for (n=iface_count() - 1; n >= 0; n--) {
 		struct interface *iface = get_interface(n);
 
+		if (!iface) {
+			DEBUG(2,("reload_interfaces: failed to get interface %d\n", n));
+			continue;
+		}
+
 		/*
 		 * We don't want to add a loopback interface, in case
 		 * someone has added 127.0.0.1 for smbd, nmbd needs to
@@ -257,15 +228,35 @@ static BOOL reload_interfaces(time_t t)
 			close_subnet(subrec);
 		}
 	}
-	
+
 	rescan_listen_set = True;
 
-	/* We need to shutdown if there are no subnets... */
+	/* We need to wait if there are no subnets... */
 	if (FIRST_SUBNET == NULL) {
-		DEBUG(0,("reload_interfaces: No subnets to listen to. Shutting down...\n"));
-		return True;
+		void (*saved_handler)(int);
+
+		DEBUG(0,("reload_interfaces: "
+			"No subnets to listen to. Waiting..\n"));
+
+		/*
+		 * Whilst we're waiting for an interface, allow SIGTERM to
+		 * cause us to exit.
+		 */
+
+		saved_handler = CatchSignal( SIGTERM, SIGNAL_CAST SIG_DFL );
+
+                while (iface_count() == 0) {
+			sleep(5);
+			load_interfaces();
+		}
+
+		/*
+		 * We got an interface, restore our normal term handler.
+		 */
+
+		CatchSignal( SIGTERM, SIGNAL_CAST saved_handler );
+		goto try_again;
 	}
-	return False;
 }
 
 /**************************************************************************** **
@@ -290,7 +281,7 @@ static BOOL reload_nmbd_services(BOOL test)
 	if ( test && !lp_file_list_changed() )
 		return(True);
 
-	ret = lp_load( dyn_CONFIGFILE, True , False, False);
+	ret = lp_load( dyn_CONFIGFILE, True , False, False, True);
 
 	/* perhaps the config filename is now set */
 	if ( !test ) {
@@ -303,51 +294,45 @@ static BOOL reload_nmbd_services(BOOL test)
 
 /**************************************************************************** **
  * React on 'smbcontrol nmbd reload-config' in the same way as to SIGHUP
- * We use buf here to return BOOL result to process() when reload_interfaces()
- * detects that there are no subnets.
  **************************************************************************** */
 
-static void msg_reload_nmbd_services(int msg_type, pid_t src, void *buf, size_t len)
+static void msg_reload_nmbd_services(int msg_type, struct process_id src,
+				     void *buf, size_t len, void *private_data)
 {
 	write_browse_list( 0, True );
 	dump_all_namelists();
 	reload_nmbd_services( True );
 	reopen_logs();
-	
-	if(buf) {
-		/* We were called from process() */
-		/* If reload_interfaces() returned True */
-		/* we need to shutdown if there are no subnets... */
-		/* pass this info back to process() */
-		*((BOOL*)buf) = reload_interfaces(0);  
-	}
+	reload_interfaces(0);
 }
 
-static void msg_nmbd_send_packet(int msg_type, pid_t src,
-				 void *buf, size_t len)
+static void msg_nmbd_send_packet(int msg_type, struct process_id src,
+				 void *buf, size_t len, void *private_data)
 {
 	struct packet_struct *p = (struct packet_struct *)buf;
 	struct subnet_record *subrec;
 	struct in_addr *local_ip;
 
-	DEBUG(10, ("Received send_packet from %d\n", src));
+	DEBUG(10, ("Received send_packet from %d\n", procid_to_pid(&src)));
 
 	if (len != sizeof(struct packet_struct)) {
-		DEBUG(2, ("Discarding invalid packet length from %d\n", src));
+		DEBUG(2, ("Discarding invalid packet length from %d\n",
+			  procid_to_pid(&src)));
 		return;
 	}
 
 	if ((p->packet_type != NMB_PACKET) &&
 	    (p->packet_type != DGRAM_PACKET)) {
 		DEBUG(2, ("Discarding invalid packet type from %d: %d\n",
-			  src, p->packet_type));
+			  procid_to_pid(&src), p->packet_type));
 		return;
 	}
 
 	local_ip = iface_ip(p->ip);
 
 	if (local_ip == NULL) {
-		DEBUG(2, ("Could not find ip for packet from %d\n", src));
+		DEBUG(2, ("Could not find ip for packet from %d\n",
+			  procid_to_pid(&src)));
 		return;
 	}
 
@@ -381,7 +366,6 @@ static void msg_nmbd_send_packet(int msg_type, pid_t src,
 static void process(void)
 {
 	BOOL run_election;
-	BOOL no_subnets;
 
 	while( True ) {
 		time_t t = time(NULL);
@@ -590,19 +574,17 @@ static void process(void)
 
 		if(reload_after_sighup) {
 			DEBUG( 0, ( "Got SIGHUP dumping debug info.\n" ) );
-			msg_reload_nmbd_services(MSG_SMB_CONF_UPDATED, (pid_t) 0, (void*) &no_subnets, 0);
-			if(no_subnets)
-				return;
+			msg_reload_nmbd_services(MSG_SMB_CONF_UPDATED,
+						 pid_to_procid(0), NULL, 0, NULL);
 			reload_after_sighup = 0;
 		}
 
 		/* check for new network interfaces */
 
-		if(reload_interfaces(t))
-			return;
+		reload_interfaces(t);
 
 		/* free up temp memory */
-			lp_talloc_free();
+		lp_TALLOC_FREE();
 	}
 }
 
@@ -627,7 +609,9 @@ static BOOL open_sockets(BOOL isdaemon, int port)
 	else
 		ClientNMB = 0;
   
-	ClientDGRAM = open_socket_in(SOCK_DGRAM,DGRAM_PORT,3,0,True);
+	ClientDGRAM = open_socket_in(SOCK_DGRAM, DGRAM_PORT,
+					   3, interpret_addr(lp_socket_address()),
+					   True);
 
 	if ( ClientNMB == -1 )
 		return( False );
@@ -638,6 +622,10 @@ static BOOL open_sockets(BOOL isdaemon, int port)
 	set_socket_options( ClientNMB,   "SO_BROADCAST" );
 	set_socket_options( ClientDGRAM, "SO_BROADCAST" );
 
+	/* Ensure we're non-blocking. */
+	set_blocking( ClientNMB, False);
+	set_blocking( ClientDGRAM, False);
+
 	DEBUG( 3, ( "open_sockets: Broadcast sockets opened.\n" ) );
 	return( True );
 }
@@ -650,23 +638,27 @@ static BOOL open_sockets(BOOL isdaemon, int port)
 	pstring logfile;
 	static BOOL opt_interactive;
 	poptContext pc;
-	int opt;
+	static char *p_lmhosts = dyn_LMHOSTSFILE;
+	static BOOL no_process_group = False;
 	struct poptOption long_options[] = {
 	POPT_AUTOHELP
 	{"daemon", 'D', POPT_ARG_VAL, &is_daemon, True, "Become a daemon(default)" },
 	{"interactive", 'i', POPT_ARG_VAL, &opt_interactive, True, "Run interactive (not a daemon)" },
 	{"foreground", 'F', POPT_ARG_VAL, &Fork, False, "Run daemon in foreground (for daemontools & etc)" },
+	{"no-process-group", 0, POPT_ARG_VAL, &no_process_group, True, "Don't create a new process group" },
 	{"log-stdout", 'S', POPT_ARG_VAL, &log_stdout, True, "Log to stdout" },
-	{"hosts", 'H', POPT_ARG_STRING, dyn_LMHOSTSFILE, 'H', "Load a netbios hosts file"},
+	{"hosts", 'H', POPT_ARG_STRING, &p_lmhosts, 'H', "Load a netbios hosts file"},
 	{"port", 'p', POPT_ARG_INT, &global_nmb_port, NMB_PORT, "Listen on the specified port" },
 	POPT_COMMON_SAMBA
 	{ NULL }
 	};
 
+	load_case_tables();
+
 	global_nmb_port = NMB_PORT;
 
 	pc = poptGetContext("nmbd", argc, argv, long_options, 0);
-	while ((opt = poptGetNextOpt(pc)) != -1) ;
+	while (poptGetNextOpt(pc) != -1) {};
 	poptFreeContext(pc);
 
 	global_in_nmbd = True;
@@ -681,6 +673,7 @@ static BOOL open_sockets(BOOL isdaemon, int port)
 	}
 	
 	fault_setup((void (*)(void *))fault_continue );
+	dump_core_setup("nmbd");
 	
 	/* POSIX demands that signals are inherited. If the invoking process has
 	 * these signals masked, we will have problems, as we won't receive them. */
@@ -716,7 +709,7 @@ static BOOL open_sockets(BOOL isdaemon, int port)
 	reopen_logs();
 
 	DEBUG( 0, ( "Netbios nameserver version %s started.\n", SAMBA_VERSION_STRING) );
-	DEBUGADD( 0, ( "Copyright Andrew Tridgell and the Samba Team 1994-2004\n" ) );
+	DEBUGADD( 0, ( "%s\n", COPYRIGHT_STARTUP_MESSAGE ) );
 
 	if ( !reload_nmbd_services(False) )
 		return(-1);
@@ -740,7 +733,7 @@ static BOOL open_sockets(BOOL isdaemon, int port)
   
 	if (is_daemon && !opt_interactive) {
 		DEBUG( 2, ( "Becoming a daemon.\n" ) );
-		become_daemon(Fork);
+		become_daemon(Fork, no_process_group);
 	}
 
 #if HAVE_SETPGID
@@ -748,7 +741,7 @@ static BOOL open_sockets(BOOL isdaemon, int port)
 	 * If we're interactive we want to set our own process group for 
 	 * signal management.
 	 */
-	if (opt_interactive)
+	if (opt_interactive && !no_process_group)
 		setpgid( (pid_t)0, (pid_t)0 );
 #endif
 
@@ -766,11 +759,16 @@ static BOOL open_sockets(BOOL isdaemon, int port)
 
 	pidfile_create("nmbd");
 	message_init();
-	message_register(MSG_FORCE_ELECTION, nmbd_message_election);
-	message_register(MSG_WINS_NEW_ENTRY, nmbd_wins_new_entry);
-	message_register(MSG_SHUTDOWN, nmbd_terminate);
-	message_register(MSG_SMB_CONF_UPDATED, msg_reload_nmbd_services);
-	message_register(MSG_SEND_PACKET, msg_nmbd_send_packet);
+	message_register(MSG_FORCE_ELECTION, nmbd_message_election, NULL);
+#if 0
+	/* Until winsrepl is done. */
+	message_register(MSG_WINS_NEW_ENTRY, nmbd_wins_new_entry, NULL);
+#endif
+	message_register(MSG_SHUTDOWN, nmbd_terminate, NULL);
+	message_register(MSG_SMB_CONF_UPDATED, msg_reload_nmbd_services, NULL);
+	message_register(MSG_SEND_PACKET, msg_nmbd_send_packet, NULL);
+
+	TimeInit();
 
 	DEBUG( 3, ( "Opening sockets %d\n", global_nmb_port ) );
 
@@ -790,8 +788,8 @@ static BOOL open_sockets(BOOL isdaemon, int port)
 	}
 
 	/* Load in any static local names. */ 
-	load_lmhosts_file(dyn_LMHOSTSFILE);
-	DEBUG(3,("Loaded hosts file %s\n", dyn_LMHOSTSFILE));
+	load_lmhosts_file(p_lmhosts);
+	DEBUG(3,("Loaded hosts file %s\n", p_lmhosts));
 
 	/* If we are acting as a WINS server, initialise data structures. */
 	if( !initialise_wins() ) {
