@@ -25,6 +25,8 @@
 
 extern SIG_ATOMIC_T got_sig_term;
 extern SIG_ATOMIC_T reload_after_sighup;
+extern struct current_user current_user;
+extern userdom_struct current_user_info;
 
 /* Current printer interface */
 static BOOL remove_from_jobs_changed(const char* sharename, uint32 jobid);
@@ -91,7 +93,7 @@ uint16 pjobid_to_rap(const char* sharename, uint32 jobid)
 	if (rap_jobid == 0)
 		rap_jobid = ++next_rap_jobid;
 	SSVAL(buf,0,rap_jobid);
-	data.dptr = buf;
+	data.dptr = (char*)buf;
 	data.dsize = sizeof(rap_jobid);
 	tdb_store(rap_tdb, key, data, TDB_REPLACE);
 	tdb_store(rap_tdb, data, key, TDB_REPLACE);
@@ -112,7 +114,7 @@ BOOL rap_to_pjobid(uint16 rap_jobid, fstring sharename, uint32 *pjobid)
 		return False;
 
 	SSVAL(buf,0,rap_jobid);
-	key.dptr = buf;
+	key.dptr = (char*)buf;
 	key.dsize = sizeof(rap_jobid);
 	data = tdb_fetch(rap_tdb, key);
 	if ( data.dptr && data.dsize == sizeof(struct rap_jobid_key) ) 
@@ -164,7 +166,7 @@ static void rap_jobid_delete(const char* sharename, uint32 jobid)
 	rap_jobid = SVAL(data.dptr, 0);
 	SAFE_FREE(data.dptr);
 	SSVAL(buf,0,rap_jobid);
-	data.dptr=buf;
+	data.dptr = (char*)buf;
 	data.dsize = sizeof(rap_jobid);
 	tdb_delete(rap_tdb, key);
 	tdb_delete(rap_tdb, data);
@@ -197,7 +199,7 @@ BOOL print_backend_init(void)
 		pdb = get_print_db_byname(lp_const_servicename(snum));
 		if (!pdb)
 			continue;
-		if (tdb_lock_bystring(pdb->tdb, sversion, 0) == -1) {
+		if (tdb_lock_bystring(pdb->tdb, sversion) == -1) {
 			DEBUG(0,("print_backend_init: Failed to open printer %s database\n", lp_const_servicename(snum) ));
 			release_print_db(pdb);
 			return False;
@@ -234,7 +236,7 @@ void printing_end(void)
  when asked for (and only when supported)
 ****************************************************************************/
 
-static struct printif *get_printer_fns_from_type( int type )
+static struct printif *get_printer_fns_from_type( enum printing_types type )
 {
 	struct printif *printer_fns = &generic_printif;
 
@@ -244,6 +246,12 @@ static struct printif *get_printer_fns_from_type( int type )
 	}
 #endif /* HAVE_CUPS */
 
+#ifdef HAVE_IPRINT
+	if ( type == PRINT_IPRINT ) {
+		printer_fns = &iprint_printif;
+	}
+#endif /* HAVE_IPRINT */
+
 	printer_fns->type = type;
 	
 	return printer_fns;
@@ -251,7 +259,7 @@ static struct printif *get_printer_fns_from_type( int type )
 
 static struct printif *get_printer_fns( int snum )
 {
-	return get_printer_fns_from_type( lp_printing(snum) );
+	return get_printer_fns_from_type( (enum printing_types)lp_printing(snum) );
 }
 
 
@@ -265,7 +273,7 @@ static TDB_DATA print_key(uint32 jobid)
 	TDB_DATA ret;
 
 	SIVAL(&j, 0, jobid);
-	ret.dptr = (void *)&j;
+	ret.dptr = (char *)&j;
 	ret.dsize = sizeof(j);
 	return ret;
 }
@@ -331,27 +339,38 @@ static struct printjob *print_job_find(const char *sharename, uint32 jobid)
 	TDB_DATA 		ret;
 	struct tdb_print_db 	*pdb = get_print_db_byname(sharename);
 	
+	DEBUG(10,("print_job_find: looking up job %u for share %s\n",
+			(unsigned int)jobid, sharename ));
 
-	if (!pdb)
+	if (!pdb) {
 		return NULL;
+	}
 
 	ret = tdb_fetch(pdb->tdb, print_key(jobid));
 	release_print_db(pdb);
 
-	if (!ret.dptr)
+	if (!ret.dptr) {
+		DEBUG(10,("print_job_find: failed to find jobid %u.\n", (unsigned int)jobid ));
 		return NULL;
+	}
 	
-	if ( pjob.nt_devmode )
+	if ( pjob.nt_devmode ) {
 		free_nt_devicemode( &pjob.nt_devmode );
+	}
 		
 	ZERO_STRUCT( pjob );
 	
 	if ( unpack_pjob( ret.dptr, ret.dsize, &pjob ) == -1 ) {
+		DEBUG(10,("print_job_find: failed to unpack jobid %u.\n", (unsigned int)jobid ));
 		SAFE_FREE(ret.dptr);
 		return NULL;
 	}
 	
-	SAFE_FREE(ret.dptr);	
+	SAFE_FREE(ret.dptr);
+
+	DEBUG(10,("print_job_find: returning system job %d for jobid %u.\n",
+			(int)pjob.sysjob, (unsigned int)jobid ));
+
 	return &pjob;
 }
 
@@ -535,15 +554,11 @@ static BOOL pjob_store(const char* sharename, uint32 jobid, struct printjob *pjo
 		len += pack_devicemode(pjob->nt_devmode, buf+len, buflen-len);
 	
 		if (buflen != len) {
-			char *tb;
-
-			tb = (char *)SMB_REALLOC(buf, len);
-			if (!tb) {
+			buf = (char *)SMB_REALLOC(buf, len);
+			if (!buf) {
 				DEBUG(0,("pjob_store: failed to enlarge buffer!\n"));
 				goto done;
 			}
-			else 
-				buf = tb;
 			newlen = len;
 		}
 	} while ( buflen != len );
@@ -685,6 +700,8 @@ struct traverse_struct {
 	int qcount, snum, maxcount, total_jobs;
 	const char *sharename;
 	time_t lpq_time;
+	const char *lprm_command;
+	struct printif *print_if;
 };
 
 /****************************************************************************
@@ -731,7 +748,7 @@ static int traverse_fn_delete(TDB_CONTEXT *t, TDB_DATA key, TDB_DATA data, void
 		/* if a job is not spooled and the process doesn't
                    exist then kill it. This cleans up after smbd
                    deaths */
-		if (!process_exists(pjob.pid)) {
+		if (!process_exists_by_pid(pjob.pid)) {
 			DEBUG(10,("traverse_fn_delete: pjob %u deleted due to !process_exists (%u)\n",
 						(unsigned int)jobid, (unsigned int)pjob.pid ));
 			pjob_delete(ts->sharename, jobid);
@@ -744,9 +761,38 @@ static int traverse_fn_delete(TDB_CONTEXT *t, TDB_DATA key, TDB_DATA data, void
 	
 	if ( pjob.smbjob ) {
 		for (i=0;i<ts->qcount;i++) {
-			uint32 curr_jobid = print_parse_jobid(ts->queue[i].fs_file);
-			if (jobid == curr_jobid)
+			uint32 curr_jobid;
+
+			if ( pjob.status == LPQ_DELETED )
+				continue;
+
+			curr_jobid = print_parse_jobid(ts->queue[i].fs_file);
+
+			if (jobid == curr_jobid) {
+
+				/* try to clean up any jobs that need to be deleted */
+
+				if ( pjob.status == LPQ_DELETING ) {
+					int result;
+
+					result = (*(ts->print_if->job_delete))( 
+						ts->sharename, ts->lprm_command, &pjob );
+
+					if ( result != 0 ) {
+						/* if we can't delete, then reset the job status */
+						pjob.status = LPQ_QUEUED;
+						pjob_store(ts->sharename, jobid, &pjob);
+					}
+					else {
+						/* if we deleted the job, the remove the tdb record */
+						pjob_delete(ts->sharename, jobid);
+						pjob.status = LPQ_DELETED;
+					}
+						
+				}
+
 				break;
+			}
 		}
 	}
 	
@@ -773,9 +819,10 @@ static int traverse_fn_delete(TDB_CONTEXT *t, TDB_DATA key, TDB_DATA data, void
 		return 0;
 	}
 
-	/* Save the pjob attributes we will store. */
-	/* FIXME!!! This is the only place where queue->job 
+	/* Save the pjob attributes we will store. 
+	   FIXME!!! This is the only place where queue->job 
 	   represents the SMB jobid      --jerry */
+
 	ts->queue[i].job = jobid;		
 	ts->queue[i].size = pjob.size;
 	ts->queue[i].page_count = pjob.page_count;
@@ -794,10 +841,9 @@ static int traverse_fn_delete(TDB_CONTEXT *t, TDB_DATA key, TDB_DATA data, void
  Check if the print queue has been updated recently enough.
 ****************************************************************************/
 
-static void print_cache_flush(int snum)
+static void print_cache_flush(const char *sharename)
 {
 	fstring key;
-	const char *sharename = lp_const_servicename(snum);
 	struct tdb_print_db *pdb = get_print_db_byname(sharename);
 
 	if (!pdb)
@@ -834,7 +880,7 @@ static pid_t get_updating_pid(const char *sharename)
 	updating_pid = IVAL(data.dptr, 0);
 	SAFE_FREE(data.dptr);
 
-	if (process_exists(updating_pid))
+	if (process_exists_by_pid(updating_pid))
 		return updating_pid;
 
 	return (pid_t)-1;
@@ -873,7 +919,7 @@ static void set_updating_pid(const fstring sharename, BOOL updating)
 	}
 	
 	SIVAL( buffer, 0, updating_pid);
-	data.dptr = (void *)buffer;
+	data.dptr = (char *)buffer;
 	data.dsize = 4;		/* we always assume this is a 4 byte value */
 
 	tdb_store(pdb->tdb, key, data, TDB_REPLACE);	
@@ -904,6 +950,7 @@ static int printjob_comp(print_queue_struct *j1, print_queue_struct *j2)
 
 /****************************************************************************
  Store the sorted queue representation for later portmon retrieval.
+ Skip deleted jobs
 ****************************************************************************/
 
 static void store_queue_struct(struct tdb_print_db *pdb, struct traverse_struct *pts)
@@ -917,13 +964,17 @@ static void store_queue_struct(struct tdb_print_db *pdb, struct traverse_struct
 
 	if (max_reported_jobs && (max_reported_jobs < pts->qcount))
 		pts->qcount = max_reported_jobs;
-	qcount = pts->qcount;
+	qcount = 0;
 
 	/* Work out the size. */
 	data.dsize = 0;
 	data.dsize += tdb_pack(NULL, 0, "d", qcount);
 
 	for (i = 0; i < pts->qcount; i++) {
+		if ( queue[i].status == LPQ_DELETED )
+			continue;
+
+		qcount++;
 		data.dsize += tdb_pack(NULL, 0, "ddddddff",
 				(uint32)queue[i].job,
 				(uint32)queue[i].size,
@@ -935,12 +986,15 @@ static void store_queue_struct(struct tdb_print_db *pdb, struct traverse_struct
 				queue[i].fs_file);
 	}
 
-	if ((data.dptr = SMB_MALLOC(data.dsize)) == NULL)
+	if ((data.dptr = (char *)SMB_MALLOC(data.dsize)) == NULL)
 		return;
 
         len = 0;
 	len += tdb_pack(data.dptr + len, data.dsize - len, "d", qcount);
 	for (i = 0; i < pts->qcount; i++) {
+		if ( queue[i].status == LPQ_DELETED )
+			continue;
+
 		len += tdb_pack(data.dptr + len, data.dsize - len, "ddddddff",
 				(uint32)queue[i].job,
 				(uint32)queue[i].size,
@@ -1018,6 +1072,7 @@ static BOOL print_cache_expired(const char *sharename, BOOL check_pending)
 		|| (time_now - last_qscan_time) >= lp_lpqcachetime() 
 		|| last_qscan_time > (time_now + MAX_CACHE_VALID_TIME)) 
 	{
+		uint32 u;
 		time_t msg_pending_time;
 
 		DEBUG(4, ("print_cache_expired: cache expired for queue %s " 
@@ -1033,8 +1088,8 @@ static BOOL print_cache_expired(const char *sharename, BOOL check_pending)
 		snprintf(key, sizeof(key), "MSG_PENDING/%s", sharename);
 
 		if ( check_pending 
-			&& tdb_fetch_uint32( pdb->tdb, key, (uint32*)&msg_pending_time ) 
-			&& msg_pending_time > 0
+			&& tdb_fetch_uint32( pdb->tdb, key, &u ) 
+			&& (msg_pending_time=u) > 0
 			&& msg_pending_time <= time_now 
 			&& (time_now - msg_pending_time) < 60 ) 
 		{
@@ -1057,7 +1112,7 @@ done:
 
 static void print_queue_update_internal( const char *sharename, 
                                          struct printif *current_printif,
-                                         char *lpq_command )
+                                         char *lpq_command, char *lprm_command )
 {
 	int i, qcount;
 	print_queue_struct *queue = NULL;
@@ -1135,8 +1190,14 @@ static void print_queue_update_internal( const char *sharename,
 		}
 
 		pjob->sysjob = queue[i].job;
-		pjob->status = queue[i].status;
+
+		/* don't reset the status on jobs to be deleted */
+
+		if ( pjob->status != LPQ_DELETING )
+			pjob->status = queue[i].status;
+
 		pjob_store(sharename, jobid, pjob);
+
 		check_job_changed(sharename, jcdata, jobid);
 	}
 
@@ -1150,6 +1211,8 @@ static void print_queue_update_internal( const char *sharename,
 	tstruct.total_jobs = 0;
 	tstruct.lpq_time = time(NULL);
 	tstruct.sharename = sharename;
+	tstruct.lprm_command = lprm_command;
+	tstruct.print_if = current_printif;
 
 	tdb_traverse(pdb->tdb, traverse_fn_delete, (void *)&tstruct);
 
@@ -1174,7 +1237,7 @@ static void print_queue_update_internal( const char *sharename,
 	key.dsize = strlen(keystr);
 
 	status.qcount = qcount;
-	data.dptr = (void *)&status;
+	data.dptr = (char *)&status;
 	data.dsize = sizeof(status);
 	tdb_store(pdb->tdb, key, data, TDB_REPLACE);	
 
@@ -1210,7 +1273,7 @@ static void print_queue_update_internal( const char *sharename,
 
 static void print_queue_update_with_lock( const char *sharename, 
                                           struct printif *current_printif,
-                                          char *lpq_command )
+                                          char *lpq_command, char *lprm_command )
 {
 	fstring keystr;
 	struct tdb_print_db *pdb;
@@ -1240,7 +1303,7 @@ static void print_queue_update_with_lock( const char *sharename,
 
 	slprintf(keystr, sizeof(keystr) - 1, "LOCK/%s", sharename);
 	/* Only wait 10 seconds for this. */
-	if (tdb_lock_bystring(pdb->tdb, keystr, 10) == -1) {
+	if (tdb_lock_bystring_with_timeout(pdb->tdb, keystr, 10) == -1) {
 		DEBUG(0,("print_queue_update_with_lock: Failed to lock printer %s database\n", sharename));
 		release_print_db(pdb);
 		return;
@@ -1277,7 +1340,8 @@ static void print_queue_update_with_lock( const char *sharename,
 
 	/* do the main work now */
 	
-	print_queue_update_internal( sharename, current_printif, lpq_command );
+	print_queue_update_internal( sharename, current_printif, 
+		lpq_command, lprm_command );
 	
 	/* Delete our pid from the db. */
 	set_updating_pid(sharename, False);
@@ -1287,17 +1351,20 @@ static void print_queue_update_with_lock( const char *sharename,
 /****************************************************************************
 this is the receive function of the background lpq updater
 ****************************************************************************/
-static void print_queue_receive(int msg_type, pid_t src, void *buf, size_t msglen)
+static void print_queue_receive(int msg_type, struct process_id src,
+				void *buf, size_t msglen,
+				void *private_data)
 {
 	fstring sharename;
-	pstring lpqcommand;
+	pstring lpqcommand, lprmcommand;
 	int printing_type;
 	size_t len;
 
-	len = tdb_unpack( buf, msglen, "fdP",
+	len = tdb_unpack( (char *)buf, msglen, "fdPP",
 		sharename,
 		&printing_type,
-		lpqcommand );
+		lpqcommand,
+		lprmcommand );
 
 	if ( len == -1 ) {
 		DEBUG(0,("print_queue_receive: Got invalid print queue update message\n"));
@@ -1305,8 +1372,8 @@ static void print_queue_receive(int msg_type, pid_t src, void *buf, size_t msgle
 	}
 
 	print_queue_update_with_lock(sharename, 
-		get_printer_fns_from_type(printing_type),
-		lpqcommand );
+		get_printer_fns_from_type((enum printing_types)printing_type),
+		lpqcommand, lprmcommand );
 
 	return;
 }
@@ -1337,7 +1404,8 @@ void start_background_queue(void)
 			exit(1);
 		}
 
-		message_register(MSG_PRINTER_UPDATE, print_queue_receive);
+		message_register(MSG_PRINTER_UPDATE, print_queue_receive,
+				 NULL);
 		
 		DEBUG(5,("start_background_queue: background LPQ thread waiting for messages\n"));
 		while (1) {
@@ -1346,7 +1414,7 @@ void start_background_queue(void)
 			/* check for some essential signals first */
 			
                         if (got_sig_term) {
-                                exit_server("Caught TERM signal");
+                                exit_server_cleanly(NULL);
                         }
 
                         if (reload_after_sighup) {
@@ -1376,7 +1444,7 @@ static void print_queue_update(int snum, BOOL force)
 {
 	fstring key;
 	fstring sharename;
-	pstring lpqcommand;
+	pstring lpqcommand, lprmcommand;
 	char *buffer = NULL;
 	size_t len = 0;
 	size_t newlen;
@@ -1389,8 +1457,24 @@ static void print_queue_update(int snum, BOOL force)
 	/* don't strip out characters like '$' from the printername */
 	
 	pstrcpy( lpqcommand, lp_lpqcommand(snum));
-	string_sub2( lpqcommand, "%p", PRINTERNAME(snum), sizeof(lpqcommand), False, False );
-	standard_sub_snum( snum, lpqcommand, sizeof(lpqcommand) );
+	string_sub2( lpqcommand, "%p", PRINTERNAME(snum), sizeof(lpqcommand), 
+		     False, False, False );
+	standard_sub_advanced(lp_servicename(snum),
+			      current_user_info.unix_name, "",
+			      current_user.ut.gid,
+			      get_current_username(),
+			      current_user_info.domain,
+			      lpqcommand, sizeof(lpqcommand) );
+	
+	pstrcpy( lprmcommand, lp_lprmcommand(snum));
+	string_sub2( lprmcommand, "%p", PRINTERNAME(snum), sizeof(lprmcommand), 
+		     False, False, False );
+	standard_sub_advanced(lp_servicename(snum),
+			      current_user_info.unix_name, "",
+			      current_user.ut.gid,
+			      get_current_username(),
+			      current_user_info.domain,
+			      lprmcommand, sizeof(lprmcommand) );
 	
 	/* 
 	 * Make sure that the background queue process exists.  
@@ -1400,7 +1484,7 @@ static void print_queue_update(int snum, BOOL force)
 	if ( force || background_lpq_updater_pid == -1 ) {
 		DEBUG(4,("print_queue_update: updating queue [%s] myself\n", sharename));
 		current_printif = get_printer_fns( snum );
-		print_queue_update_with_lock( sharename, current_printif, lpqcommand );
+		print_queue_update_with_lock( sharename, current_printif, lpqcommand, lprmcommand );
 
 		return;
 	}
@@ -1409,23 +1493,26 @@ static void print_queue_update(int snum, BOOL force)
 	
 	/* get the length */
 
-	len = tdb_pack( buffer, len, "fdP",
+	len = tdb_pack( NULL, 0, "fdPP",
 		sharename,
 		type,
-		lpqcommand );
+		lpqcommand, 
+		lprmcommand );
 
 	buffer = SMB_XMALLOC_ARRAY( char, len );
 
 	/* now pack the buffer */
-	newlen = tdb_pack( buffer, len, "fdP",
+	newlen = tdb_pack( buffer, len, "fdPP",
 		sharename,
 		type,
-		lpqcommand );
+		lpqcommand,
+		lprmcommand );
 
 	SMB_ASSERT( newlen == len );
 
 	DEBUG(10,("print_queue_update: Sending message -> printer = %s, "
-		"type = %d, lpq command = [%s]\n", sharename, type, lpqcommand ));
+		"type = %d, lpq command = [%s] lprm command = [%s]\n", 
+		sharename, type, lpqcommand, lprmcommand ));
 
 	/* here we set a msg pending record for other smbd processes 
 	   to throttle the number of duplicate print_queue_update msgs
@@ -1450,10 +1537,8 @@ static void print_queue_update(int snum, BOOL force)
 
 	/* finally send the message */
 	
-	become_root();
-	message_send_pid(background_lpq_updater_pid,
+	message_send_pid(pid_to_procid(background_lpq_updater_pid),
 		 MSG_PRINTER_UPDATE, buffer, len, False);
-	unbecome_root();
 
 	SAFE_FREE( buffer );
 
@@ -1500,7 +1585,7 @@ BOOL print_notify_register_pid(int snum)
 		tdb = pdb->tdb;
 	}
 
-	if (tdb_lock_bystring(tdb, NOTIFY_PID_LIST_KEY, 10) == -1) {
+	if (tdb_lock_bystring_with_timeout(tdb, NOTIFY_PID_LIST_KEY, 10) == -1) {
 		DEBUG(0,("print_notify_register_pid: Failed to lock printer %s\n",
 					printername));
 		if (pdb)
@@ -1522,7 +1607,7 @@ BOOL print_notify_register_pid(int snum)
 
 	if (i == data.dsize) {
 		/* We weren't in the list. Realloc. */
-		data.dptr = SMB_REALLOC(data.dptr, data.dsize + 8);
+		data.dptr = (char *)SMB_REALLOC(data.dptr, data.dsize + 8);
 		if (!data.dptr) {
 			DEBUG(0,("print_notify_register_pid: Relloc fail for printer %s\n",
 						printername));
@@ -1590,7 +1675,7 @@ BOOL print_notify_deregister_pid(int snum)
 		tdb = pdb->tdb;
 	}
 
-	if (tdb_lock_bystring(tdb, NOTIFY_PID_LIST_KEY, 10) == -1) {
+	if (tdb_lock_bystring_with_timeout(tdb, NOTIFY_PID_LIST_KEY, 10) == -1) {
 		DEBUG(0,("print_notify_register_pid: Failed to lock \
 printer %s database\n", printername));
 		if (pdb)
@@ -1707,7 +1792,7 @@ NT_DEVICEMODE *print_job_devmode(const char* sharename, uint32 jobid)
  Set the place in the queue for a job.
 ****************************************************************************/
 
-BOOL print_job_set_place(int snum, uint32 jobid, int place)
+BOOL print_job_set_place(const char *sharename, uint32 jobid, int place)
 {
 	DEBUG(2,("print_job_set_place not implemented yet\n"));
 	return False;
@@ -1717,9 +1802,8 @@ BOOL print_job_set_place(int snum, uint32 jobid, int place)
  Set the name of a job. Only possible for owner.
 ****************************************************************************/
 
-BOOL print_job_set_name(int snum, uint32 jobid, char *name)
+BOOL print_job_set_name(const char *sharename, uint32 jobid, char *name)
 {
-	const char* sharename = lp_const_servicename(snum);
 	struct printjob *pjob;
 
 	pjob = print_job_find(sharename, jobid);
@@ -1800,8 +1884,6 @@ static BOOL print_job_delete1(int snum, uint32 jobid)
 	int result = 0;
 	struct printif *current_printif = get_printer_fns( snum );
 
-	pjob = print_job_find(sharename, jobid);
-
 	if (!pjob)
 		return False;
 
@@ -1813,7 +1895,9 @@ static BOOL print_job_delete1(int snum, uint32 jobid)
 		return True;
 
 	/* Hrm - we need to be able to cope with deleting a job before it
-	   has reached the spooler. */
+	   has reached the spooler.  Just mark it as LPQ_DELETING and 
+	   let the print_queue_update() code rmeove the record */
+	   
 
 	if (pjob->sysjob == -1) {
 		DEBUG(5, ("attempt to delete job %u not seen by lpr\n", (unsigned int)jobid));
@@ -1824,24 +1908,31 @@ static BOOL print_job_delete1(int snum, uint32 jobid)
 	pjob->status = LPQ_DELETING;
 	pjob_store(sharename, jobid, pjob);
 
-	if (pjob->spooled && pjob->sysjob != -1)
-		result = (*(current_printif->job_delete))(snum, pjob);
+	if (pjob->spooled && pjob->sysjob != -1) 
+	{
+		result = (*(current_printif->job_delete))(
+			PRINTERNAME(snum),
+			lp_lprmcommand(snum), 
+			pjob);
 
-	/* Delete the tdb entry if the delete succeeded or the job hasn't
-	   been spooled. */
+		/* Delete the tdb entry if the delete succeeded or the job hasn't
+		   been spooled. */
 
-	if (result == 0) {
-		struct tdb_print_db *pdb = get_print_db_byname(sharename);
-		int njobs = 1;
+		if (result == 0) {
+			struct tdb_print_db *pdb = get_print_db_byname(sharename);
+			int njobs = 1;
 
-		if (!pdb)
-			return False;
-		pjob_delete(sharename, jobid);
-		/* Ensure we keep a rough count of the number of total jobs... */
-		tdb_change_int32_atomic(pdb->tdb, "INFO/total_jobs", &njobs, -1);
-		release_print_db(pdb);
+			if (!pdb)
+				return False;
+			pjob_delete(sharename, jobid);
+			/* Ensure we keep a rough count of the number of total jobs... */
+			tdb_change_int32_atomic(pdb->tdb, "INFO/total_jobs", &njobs, -1);
+			release_print_db(pdb);
+		}
 	}
 
+	remove_from_jobs_changed( sharename, jobid );
+
 	return (result == 0);
 }
 
@@ -1849,9 +1940,10 @@ static BOOL print_job_delete1(int snum, uint32 jobid)
  Return true if the current user owns the print job.
 ****************************************************************************/
 
-static BOOL is_owner(struct current_user *user, int snum, uint32 jobid)
+static BOOL is_owner(struct current_user *user, const char *servicename,
+		     uint32 jobid)
 {
-	struct printjob *pjob = print_job_find(lp_const_servicename(snum), jobid);
+	struct printjob *pjob = print_job_find(servicename, jobid);
 	user_struct *vuser;
 
 	if (!pjob || !user)
@@ -1860,7 +1952,7 @@ static BOOL is_owner(struct current_user *user, int snum, uint32 jobid)
 	if ((vuser = get_valid_user_struct(user->vuid)) != NULL) {
 		return strequal(pjob->user, vuser->user.smb_name);
 	} else {
-		return strequal(pjob->user, uidtoname(user->uid));
+		return strequal(pjob->user, uidtoname(user->ut.uid));
 	}
 }
 
@@ -1871,12 +1963,13 @@ static BOOL is_owner(struct current_user *user, int snum, uint32 jobid)
 BOOL print_job_delete(struct current_user *user, int snum, uint32 jobid, WERROR *errcode)
 {
 	const char* sharename = lp_const_servicename( snum );
-	BOOL 	owner, deleted;
+	struct printjob *pjob;
+	BOOL 	owner;
 	char 	*fname;
 
 	*errcode = WERR_OK;
 		
-	owner = is_owner(user, snum, jobid);
+	owner = is_owner(user, lp_const_servicename(snum), jobid);
 	
 	/* Check access against security descriptor or whether the user
 	   owns their job. */
@@ -1890,7 +1983,7 @@ BOOL print_job_delete(struct current_user *user, int snum, uint32 jobid, WERROR
 		sys_adminlog( LOG_ERR, 
 			      "Permission denied-- user not allowed to delete, \
 pause, or resume print job. User name: %s. Printer name: %s.",
-			      uidtoname(user->uid), PRINTERNAME(snum) );
+			      uidtoname(user->ut.uid), PRINTERNAME(snum) );
 		/* END_ADMIN_LOG */
 
 		return False;
@@ -1923,11 +2016,11 @@ pause, or resume print job. User name: %s. Printer name: %s.",
 
 	print_queue_update(snum, True);
 	
-	deleted = !print_job_exists(sharename, jobid);
-	if ( !deleted )
+	pjob = print_job_find(sharename, jobid);
+	if ( pjob && (pjob->status != LPQ_DELETING) )
 		*errcode = WERR_ACCESS_DENIED;
 
-	return deleted;
+	return (pjob == NULL );
 }
 
 /****************************************************************************
@@ -1943,13 +2036,19 @@ BOOL print_job_pause(struct current_user *user, int snum, uint32 jobid, WERROR *
 
 	pjob = print_job_find(sharename, jobid);
 	
-	if (!pjob || !user) 
+	if (!pjob || !user) {
+		DEBUG(10, ("print_job_pause: no pjob or user for jobid %u\n",
+			(unsigned int)jobid ));
 		return False;
+	}
 
-	if (!pjob->spooled || pjob->sysjob == -1) 
+	if (!pjob->spooled || pjob->sysjob == -1) {
+		DEBUG(10, ("print_job_pause: not spooled or bad sysjob = %d for jobid %u\n",
+			(int)pjob->sysjob, (unsigned int)jobid ));
 		return False;
+	}
 
-	if (!is_owner(user, snum, jobid) &&
+	if (!is_owner(user, lp_const_servicename(snum), jobid) &&
 	    !print_access_check(user, snum, JOB_ACCESS_ADMINISTER)) {
 		DEBUG(3, ("pause denied by security descriptor\n"));
 
@@ -1957,7 +2056,7 @@ BOOL print_job_pause(struct current_user *user, int snum, uint32 jobid, WERROR *
 		sys_adminlog( LOG_ERR, 
 			"Permission denied-- user not allowed to delete, \
 pause, or resume print job. User name: %s. Printer name: %s.",
-				uidtoname(user->uid), PRINTERNAME(snum) );
+				uidtoname(user->ut.uid), PRINTERNAME(snum) );
 		/* END_ADMIN_LOG */
 
 		*errcode = WERR_ACCESS_DENIED;
@@ -1973,7 +2072,7 @@ pause, or resume print job. User name: %s. Printer name: %s.",
 	}
 
 	/* force update the database */
-	print_cache_flush(snum);
+	print_cache_flush(lp_const_servicename(snum));
 
 	/* Send a printer notify message */
 
@@ -1997,13 +2096,19 @@ BOOL print_job_resume(struct current_user *user, int snum, uint32 jobid, WERROR
 
 	pjob = print_job_find(sharename, jobid);
 	
-	if (!pjob || !user)
+	if (!pjob || !user) {
+		DEBUG(10, ("print_job_resume: no pjob or user for jobid %u\n",
+			(unsigned int)jobid ));
 		return False;
+	}
 
-	if (!pjob->spooled || pjob->sysjob == -1)
+	if (!pjob->spooled || pjob->sysjob == -1) {
+		DEBUG(10, ("print_job_resume: not spooled or bad sysjob = %d for jobid %u\n",
+			(int)pjob->sysjob, (unsigned int)jobid ));
 		return False;
+	}
 
-	if (!is_owner(user, snum, jobid) &&
+	if (!is_owner(user, lp_const_servicename(snum), jobid) &&
 	    !print_access_check(user, snum, JOB_ACCESS_ADMINISTER)) {
 		DEBUG(3, ("resume denied by security descriptor\n"));
 		*errcode = WERR_ACCESS_DENIED;
@@ -2012,7 +2117,7 @@ BOOL print_job_resume(struct current_user *user, int snum, uint32 jobid, WERROR
 		sys_adminlog( LOG_ERR, 
 			 "Permission denied-- user not allowed to delete, \
 pause, or resume print job. User name: %s. Printer name: %s.",
-			uidtoname(user->uid), PRINTERNAME(snum) );
+			uidtoname(user->ut.uid), PRINTERNAME(snum) );
 		/* END_ADMIN_LOG */
 		return False;
 	}
@@ -2025,7 +2130,7 @@ pause, or resume print job. User name: %s. Printer name: %s.",
 	}
 
 	/* force update the database */
-	print_cache_flush(snum);
+	print_cache_flush(lp_const_servicename(snum));
 
 	/* Send a printer notify message */
 
@@ -2134,7 +2239,7 @@ static BOOL allocate_print_jobid(struct tdb_print_db *pdb, int snum, const char
 
 	for (i = 0; i < 3; i++) {
 		/* Lock the database - only wait 20 seconds. */
-		if (tdb_lock_bystring(pdb->tdb, "INFO/nextjob", 20) == -1) {
+		if (tdb_lock_bystring_with_timeout(pdb->tdb, "INFO/nextjob", 20) == -1) {
 			DEBUG(0,("allocate_print_jobid: failed to lock printing database %s\n", sharename));
 			return False;
 		}
@@ -2231,7 +2336,7 @@ uint32 print_job_start(struct current_user *user, int snum, char *jobname, NT_DE
 		return (uint32)-1;
 	}
 
-	if (!print_time_access_check(snum)) {
+	if (!print_time_access_check(lp_servicename(snum))) {
 		DEBUG(3, ("print_job_start: job start denied by time check\n"));
 		release_print_db(pdb);
 		return (uint32)-1;
@@ -2291,9 +2396,13 @@ uint32 print_job_start(struct current_user *user, int snum, char *jobname, NT_DE
 	fstrcpy(pjob.jobname, jobname);
 
 	if ((vuser = get_valid_user_struct(user->vuid)) != NULL) {
-		fstrcpy(pjob.user, vuser->user.smb_name);
+		fstrcpy(pjob.user, lp_printjob_username(snum));
+		standard_sub_basic(vuser->user.smb_name, vuser->user.domain, 
+				   pjob.user, sizeof(pjob.user)-1);
+		/* ensure NULL termination */ 
+		pjob.user[sizeof(pjob.user)-1] = '\0'; 
 	} else {
-		fstrcpy(pjob.user, uidtoname(user->uid));
+		fstrcpy(pjob.user, uidtoname(user->ut.uid));
 	}
 
 	fstrcpy(pjob.queuename, lp_const_servicename(snum));
@@ -2364,7 +2473,7 @@ void print_job_endpage(int snum, uint32 jobid)
  error.
 ****************************************************************************/
 
-BOOL print_job_end(int snum, uint32 jobid, BOOL normal_close)
+BOOL print_job_end(int snum, uint32 jobid, enum file_close_type close_type)
 {
 	const char* sharename = lp_const_servicename(snum);
 	struct printjob *pjob;
@@ -2380,7 +2489,8 @@ BOOL print_job_end(int snum, uint32 jobid, BOOL normal_close)
 	if (pjob->spooled || pjob->pid != sys_getpid())
 		return False;
 
-	if (normal_close && (sys_fstat(pjob->fd, &sbuf) == 0)) {
+	if ((close_type == NORMAL_CLOSE || close_type == SHUTDOWN_CLOSE) &&
+				(sys_fstat(pjob->fd, &sbuf) == 0)) {
 		pjob->size = sbuf.st_size;
 		close(pjob->fd);
 		pjob->fd = -1;
@@ -2644,7 +2754,7 @@ BOOL print_queue_pause(struct current_user *user, int snum, WERROR *errcode)
 	}
 
 	/* force update the database */
-	print_cache_flush(snum);
+	print_cache_flush(lp_const_servicename(snum));
 
 	/* Send a printer notify message */
 
@@ -2710,7 +2820,7 @@ BOOL print_queue_purge(struct current_user *user, int snum, WERROR *errcode)
 		become_root();
 
 	for (i=0;i<njobs;i++) {
-		BOOL owner = is_owner(user, snum, queue[i].job);
+		BOOL owner = is_owner(user, lp_const_servicename(snum), queue[i].job);
 
 		if (owner || can_job_admin) {
 			print_job_delete1(snum, queue[i].job);
