@@ -18,6 +18,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#define DBGC_CLASS DBGC_LOCKING
 #include "includes.h"
 
 #if HAVE_KERNEL_OPLOCKS_LINUX
@@ -88,7 +89,8 @@ static void set_capability(unsigned capability)
 	header.pid = 0;
 
 	if (capget(&header, &data) == -1) {
-		DEBUG(3,("Unable to get kernel capabilities (%s)\n", strerror(errno)));
+		DEBUG(3,("Unable to get kernel capabilities (%s)\n",
+			 strerror(errno)));
 		return;
 	}
 
@@ -100,20 +102,29 @@ static void set_capability(unsigned capability)
 	}
 }
 
+/*
+ Call to set the kernel lease signal handler
+*/
+int linux_set_lease_sighandler(int fd)
+{
+        if (fcntl(fd, F_SETSIG, RT_SIGNAL_LEASE) == -1) {
+                DEBUG(3,("Failed to set signal handler for kernel lease\n"));
+                return -1;
+        }
+
+	return 0;
+}
+
 /****************************************************************************
  Call SETLEASE. If we get EACCES then we try setting up the right capability and
- try again
+ try again.
+ Use the SMB_VFS_LINUX_SETLEASE instead of this call directly.
 ****************************************************************************/
 
-static int linux_setlease(int fd, int leasetype)
+int linux_setlease(int fd, int leasetype)
 {
 	int ret;
 
-	if (fcntl(fd, F_SETSIG, RT_SIGNAL_LEASE) == -1) {
-		DEBUG(3,("Failed to set signal handler for kernel lease\n"));
-		return -1;
-	}
-
 	ret = fcntl(fd, F_SETLEASE, leasetype);
 	if (ret == -1 && errno == EACCES) {
 		set_capability(CAP_LEASE);
@@ -128,10 +139,10 @@ static int linux_setlease(int fd, int leasetype)
  * oplock break protocol.
 ****************************************************************************/
 
-static BOOL linux_oplock_receive_message(fd_set *fds, char *buffer, int buffer_len)
+static files_struct *linux_oplock_receive_message(fd_set *fds)
 {
 	int fd;
-	struct files_struct *fsp;
+	files_struct *fsp;
 
 	BlockSignals(True, RT_SIGNAL_LEASE);
 	fd = fd_pending_array[0];
@@ -145,32 +156,7 @@ static BOOL linux_oplock_receive_message(fd_set *fds, char *buffer, int buffer_l
 	/* now we can receive more signals */
 	BlockSignals(False, RT_SIGNAL_LEASE);
 
-	if (fsp == NULL) {
-		DEBUG(0,("Invalid file descriptor %d in kernel oplock break!\n", (int)fd));
-		return False;
-	}
-
-	DEBUG(3,("linux_oplock_receive_message: kernel oplock break request received for \
-dev = %x, inode = %.0f fd = %d, fileid = %lu \n", (unsigned int)fsp->dev, (double)fsp->inode,
-			fd, fsp->file_id));
-     
-	/*
-	 * Create a kernel oplock break message.
-	 */
-     
-	/* Setup the message header */
-	SIVAL(buffer,OPBRK_CMD_LEN_OFFSET,KERNEL_OPLOCK_BREAK_MSG_LEN);
-	SSVAL(buffer,OPBRK_CMD_PORT_OFFSET,0);
-     
-	buffer += OPBRK_CMD_HEADER_LEN;
-     
-	SSVAL(buffer,OPBRK_MESSAGE_CMD_OFFSET,KERNEL_OPLOCK_BREAK_CMD);
-     
-	memcpy(buffer + KERNEL_OPLOCK_BREAK_DEV_OFFSET, (char *)&fsp->dev, sizeof(fsp->dev));
-	memcpy(buffer + KERNEL_OPLOCK_BREAK_INODE_OFFSET, (char *)&fsp->inode, sizeof(fsp->inode));	
-	memcpy(buffer + KERNEL_OPLOCK_BREAK_FILEID_OFFSET, (char *)&fsp->file_id, sizeof(fsp->file_id));	
-
-	return True;
+	return fsp;
 }
 
 /****************************************************************************
@@ -179,16 +165,19 @@ dev = %x, inode = %.0f fd = %d, fileid = %lu \n", (unsigned int)fsp->dev, (doubl
 
 static BOOL linux_set_kernel_oplock(files_struct *fsp, int oplock_type)
 {
-	if (linux_setlease(fsp->fh->fd, F_WRLCK) == -1) {
-		DEBUG(3,("linux_set_kernel_oplock: Refused oplock on file %s, fd = %d, dev = %x, \
-inode = %.0f. (%s)\n",
+	if ( SMB_VFS_LINUX_SETLEASE(fsp,fsp->fh->fd, F_WRLCK) == -1) {
+		DEBUG(3,("linux_set_kernel_oplock: Refused oplock on file %s, "
+			 "fd = %d, dev = %x, inode = %.0f. (%s)\n",
 			 fsp->fsp_name, fsp->fh->fd, 
-			 (unsigned int)fsp->dev, (double)fsp->inode, strerror(errno)));
+			 (unsigned int)fsp->dev, (double)fsp->inode,
+			 strerror(errno)));
 		return False;
 	}
 	
-	DEBUG(3,("linux_set_kernel_oplock: got kernel oplock on file %s, dev = %x, inode = %.0f, file_id = %lu\n",
-		  fsp->fsp_name, (unsigned int)fsp->dev, (double)fsp->inode, fsp->file_id));
+	DEBUG(3,("linux_set_kernel_oplock: got kernel oplock on file %s, "
+		 "dev = %x, inode = %.0f, file_id = %lu\n",
+		  fsp->fsp_name, (unsigned int)fsp->dev, (double)fsp->inode,
+		 fsp->fh->file_id));
 
 	return True;
 }
@@ -205,48 +194,27 @@ static void linux_release_kernel_oplock(files_struct *fsp)
 		 * oplock state of this file.
 		 */
 		int state = fcntl(fsp->fh->fd, F_GETLEASE, 0);
-		dbgtext("linux_release_kernel_oplock: file %s, dev = %x, inode = %.0f file_id = %lu has kernel \
-oplock state of %x.\n", fsp->fsp_name, (unsigned int)fsp->dev,
-                        (double)fsp->inode, fsp->file_id, state );
+		dbgtext("linux_release_kernel_oplock: file %s, dev = %x, "
+			"inode = %.0f file_id = %lu has kernel oplock state "
+			"of %x.\n", fsp->fsp_name, (unsigned int)fsp->dev,
+                        (double)fsp->inode, fsp->fh->file_id, state );
 	}
 
 	/*
 	 * Remove the kernel oplock on this file.
 	 */
-	if (linux_setlease(fsp->fh->fd, F_UNLCK) == -1) {
+	if ( SMB_VFS_LINUX_SETLEASE(fsp,fsp->fh->fd, F_UNLCK) == -1) {
 		if (DEBUGLVL(0)) {
-			dbgtext("linux_release_kernel_oplock: Error when removing kernel oplock on file " );
-			dbgtext("%s, dev = %x, inode = %.0f, file_id = %lu. Error was %s\n",
-				fsp->fsp_name, (unsigned int)fsp->dev, 
-				(double)fsp->inode, fsp->file_id, strerror(errno) );
+			dbgtext("linux_release_kernel_oplock: Error when "
+				"removing kernel oplock on file " );
+			dbgtext("%s, dev = %x, inode = %.0f, file_id = %lu. "
+				"Error was %s\n", fsp->fsp_name,
+				(unsigned int)fsp->dev, (double)fsp->inode,
+				fsp->fh->file_id, strerror(errno) );
 		}
 	}
 }
 
-/****************************************************************************
- Parse a kernel oplock message.
-****************************************************************************/
-
-static BOOL linux_kernel_oplock_parse(char *msg_start, int msg_len, SMB_INO_T *inode,
-		SMB_DEV_T *dev, unsigned long *file_id)
-{
-	/* Ensure that the msg length is correct. */
-	if (msg_len != KERNEL_OPLOCK_BREAK_MSG_LEN) {
-		DEBUG(0,("incorrect length for KERNEL_OPLOCK_BREAK_CMD (was %d, should be %lu).\n", 
-			 msg_len, (unsigned long)KERNEL_OPLOCK_BREAK_MSG_LEN));
-		return False;
-	}
-
-	memcpy((char *)inode, msg_start+KERNEL_OPLOCK_BREAK_INODE_OFFSET, sizeof(*inode));
-	memcpy((char *)dev, msg_start+KERNEL_OPLOCK_BREAK_DEV_OFFSET, sizeof(*dev));
-	memcpy((char *)file_id, msg_start+KERNEL_OPLOCK_BREAK_FILEID_OFFSET, sizeof(*file_id));
-
-	DEBUG(3,("kernel oplock break request for file dev = %x, inode = %.0f, file_id = %lu\n", 
-		(unsigned int)*dev, (double)*inode, *file_id));
-
-	return True;
-}
-
 /****************************************************************************
  See if a oplock message is waiting.
 ****************************************************************************/
@@ -299,7 +267,6 @@ struct kernel_oplocks *linux_init_kernel_oplocks(void)
 	koplocks.receive_message = linux_oplock_receive_message;
 	koplocks.set_oplock = linux_set_kernel_oplock;
 	koplocks.release_oplock = linux_release_kernel_oplock;
-	koplocks.parse_message = linux_kernel_oplock_parse;
 	koplocks.msg_waiting = linux_oplock_msg_waiting;
 	koplocks.notification_fd = -1;
 
