@@ -1,8 +1,10 @@
 /* 
    Unix SMB/CIFS implementation.
    session handling for utmp and PAM
-   Copyright (C) tridge@samba.org 2001
-   Copyright (C) abartlet@pcug.org.au 2001
+
+   Copyright (C) tridge@samba.org       2001
+   Copyright (C) abartlet@samba.org     2001
+   Copyright (C) Gerald (Jerry) Carter  2006   
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,13 +31,16 @@
 
 static TDB_CONTEXT *tdb;
 
+/********************************************************************
+********************************************************************/
+
 BOOL session_init(void)
 {
 	if (tdb)
 		return True;
 
-	tdb = tdb_open_ex(lock_path("sessionid.tdb"), 0, TDB_CLEAR_IF_FIRST|TDB_DEFAULT, 
-		       O_RDWR | O_CREAT, 0644, smbd_tdb_log);
+	tdb = tdb_open_log(lock_path("sessionid.tdb"), 0, TDB_CLEAR_IF_FIRST|TDB_DEFAULT, 
+		       O_RDWR | O_CREAT, 0644);
 	if (!tdb) {
 		DEBUG(1,("session_init: failed to open sessionid tdb\n"));
 		return False;
@@ -44,7 +49,10 @@ BOOL session_init(void)
 	return True;
 }
 
-/* called when a session is created */
+/********************************************************************
+ called when a session is created
+********************************************************************/
+
 BOOL session_claim(user_struct *vuser)
 {
 	int i = 0;
@@ -126,6 +134,7 @@ BOOL session_claim(user_struct *vuser)
 	sessionid.gid = vuser->gid;
 	fstrcpy(sessionid.remote_machine, get_remote_machine_name());
 	fstrcpy(sessionid.ip_addr, client_addr());
+	sessionid.connect_start = time(NULL);
 
 	client_ip = client_inaddr(&sa);
 
@@ -151,7 +160,7 @@ BOOL session_claim(user_struct *vuser)
 			       sessionid.id_str, sessionid.id_num);
 	}
 
-	vuser->session_keystr = strdup(keystr);
+	vuser->session_keystr = SMB_STRDUP(keystr);
 	if (!vuser->session_keystr) {
 		DEBUG(0, ("session_claim:  strdup() failed for session_keystr\n"));
 		return False;
@@ -159,7 +168,10 @@ BOOL session_claim(user_struct *vuser)
 	return True;
 }
 
-/* called when a session is destroyed */
+/********************************************************************
+ called when a session is destroyed
+********************************************************************/
+
 void session_yield(user_struct *vuser)
 {
 	TDB_DATA dbuf;
@@ -198,7 +210,11 @@ void session_yield(user_struct *vuser)
 	tdb_delete(tdb, key);
 }
 
-static BOOL session_traverse(int (*fn)(TDB_CONTEXT *, TDB_DATA, TDB_DATA, void *), void *state)
+/********************************************************************
+********************************************************************/
+
+BOOL session_traverse(int (*fn)(TDB_CONTEXT *, TDB_DATA, TDB_DATA, void *),
+		      void *state)
 {
 	if (!session_init()) {
 		DEBUG(3, ("No tdb opened\n"));
@@ -209,28 +225,40 @@ static BOOL session_traverse(int (*fn)(TDB_CONTEXT *, TDB_DATA, TDB_DATA, void *
 	return True;
 }
 
+/********************************************************************
+********************************************************************/
+
 struct session_list {
 	int count;
 	struct sessionid *sessions;
 };
 
-static int gather_sessioninfo(TDB_CONTEXT *stdb, TDB_DATA kbuf, TDB_DATA dbuf,
-			      void *state)
+static int gather_sessioninfo(TDB_CONTEXT *stdb, TDB_DATA kbuf, TDB_DATA dbuf, void *state)
 {
+	uint32 i;	
 	struct session_list *sesslist = (struct session_list *) state;
 	const struct sessionid *current = (const struct sessionid *) dbuf.dptr;
 
-	sesslist->count += 1;
-	sesslist->sessions = REALLOC(sesslist->sessions, sesslist->count * 
-				      sizeof(struct sessionid));
+	i = sesslist->count;
+	
+	sesslist->sessions = SMB_REALLOC_ARRAY(sesslist->sessions, struct sessionid, i+1);
+	if (!sesslist->sessions) {
+		sesslist->count = 0;
+		return -1;
+	}
+
+	memcpy(&sesslist->sessions[i], current, sizeof(struct sessionid));
+	sesslist->count++;
 
-	memcpy(&sesslist->sessions[sesslist->count - 1], current, 
-	       sizeof(struct sessionid));
 	DEBUG(7,("gather_sessioninfo session from %s@%s\n", 
 		 current->username, current->remote_machine));
+
 	return 0;
 }
 
+/********************************************************************
+********************************************************************/
+
 int list_sessions(struct sessionid **session_list)
 {
 	struct session_list sesslist;
